from .utils import *


def tsugu_bot(message, user_id, platform, channel_id):
    message = message.strip()

    # 进行车牌匹配
    status = submit_car_number_msg(message, user_id, platform)
    if status:
        return None  # 已经匹配了车牌，就不需要再匹配其他指令了

    # 进行 v2 api 命令匹配
    command_matched, api = match_command(message, load_commands_from_config(tsugu_config.commands))
    if command_matched:
        return v2_api_command(message, command_matched, api, platform, user_id, channel_id)

    if message.endswith('服玩家状态'):
        # 如果匹配 *服玩家状态 则查询对应服务器的玩家状态 前提是用户输入的 * 是一个有效的服务器名 否则返回None
        return player_status(user_id, platform, r_) if server_exists(r_ := query_server_info(message[:-4])) else text_response('服务器没找到喵') if len(message) <= 7 else None

    if message.startswith('玩家状态'):
        # 如果匹配 玩家状态 则查询默认服务器的玩家状态 如果用户输入了服务器名 则查询对应服务器的玩家状态，如果服务器名无效则返回None
        return player_status(user_id, platform) if (arg_ := message[4:].strip()) == '' else (player_status(message, user_id, platform, r_) if server_exists(r_ := query_server_info(arg_)) else None)

    if message.startswith('绑定玩家'):
        # 如果匹配 绑定玩家 则绑定默认服务器的玩家 如果用户输入了服务器名 则绑定对应服务器的玩家，如果服务器名无效则 赋值为 None
        server = get_user_data(platform, user_id)['data']['server_mode'] if message[4:].strip() == '' else (r_ if server_exists(r_ := query_server_info(message[4:])) else None)
        if not server:
            return text_response(f'未找到名为 {message[4:]} 的服务器信息，请确保您输入的是服务器名而不是玩家ID，通常情况您只需要发送"绑定玩家"或"绑定玩家 服务器"即可')
        if r := bind_player_request(platform, user_id, server, True).get('status') == 'failed':
            return text_response(r.get('data') + ' 喵')
        return text_response(f'''正在绑定账号，请将您的 评论(个性签名) 或者 当前使用的 乐队编队名称改为\n{r.get('data')['verifyCode']}\n稍等片刻等待同步后，发送\n验证 + 空格 + 您的玩家ID 来完成本次身份验证\n例如：验证 114514''')

    if message.startswith('解除绑定'):
        server = get_user_data(platform, user_id)['data']['server_mode'] if message[4:].strip() == '' else (r_ if (tsugu_config.server_list(r_ := query_server_info(message[4:]))) else None)
        if not server:
            return text_response(f'未找到名为 {message[4:].strip()} 的服务器信息，请确保您输入的是服务器名而不是玩家ID，通常情况您只需要发送"解除绑定"即可')
        # 如果是400
        if r := bind_player_request(platform, user_id, server, False).get('status') == 'failed':
            return text_response(r.get('data') + ' 喵')
        # 如果是200
        return text_response(f'''正在解除，请将您的 评论(个性签名) 或者 当前使用的 乐队编队名称改为\n{r.get('data')['verifyCode']}\n稍等片刻等待同步后，发送\n验证解绑 {server}\n来完成本次身份验证(没错只需要加上 {server} 来确定您需要解绑的服务器)''')

    if message.startswith('验证解绑'):
        if not message[4:].strip().isdigit():
            return text_response('请输入解绑时提供的serverID(数字)')
        r = bind_player_verification(platform, user_id, int(message[4:].strip()), get_user_data(platform, user_id)['data']['server_list'][int(message[4:].strip())]['playerId'], False)
        if r.get('status') == 'failed':
            return text_response(r.get('data') + ' 喵')
        return text_response('解绑成功喵！')

    if message.startswith('验证'):
        if not message[2:].strip().isdigit():
            return text_response('请输入正确的玩家ID(数字)')
        if r := bind_player_verification(platform, user_id, get_user_data(platform, user_id)['data']['server_mode'], message[2:].strip(), True).get('status') == 'failed':
            return text_response(r.get('data') + ' 喵') if r.get('data') != '错误: 未请求绑定或解除绑定玩家' else None
        return text_response('绑定成功！现在可以使用"玩家状态"来查询玩家信息了～')

    if message in ['开启车牌转发', '开启个人车牌转发']:
        r = set_car_forward(platform, user_id, True)
        return text_response('已开启车牌转发') if r.get('status') == 'success' else None

    if message in ['关闭车牌转发', '关闭个人车牌转发']:
        r = set_car_forward(platform, user_id, False)
        return text_response('已关闭车牌转发') if r.get('status') == 'success' else None

    if message.endswith('服模式'):
        if r := set_server_mode(platform, user_id, message[:-2]).get('status') == 'success':
            return text_response('已切换为' + message[:-2] + '模式')
        return text_response(r.get('data')) if len(message) <= 5 else None

    if message.startswith('主服务器'):
        if r := set_server_mode(platform, user_id, message[4:].strip()).get('status') == 'success':
            return text_response('主服务器已切换为' + message[4:].strip())
        return text_response(r.get('data')) if ' ' in message else None

    if message.startswith('设置默认服务器'):
        if r := set_default_server(platform, user_id, message[6:].strip()).get('status') == 'success':
            return text_response('默认服务器已设置为' + message[6:].strip())
        return text_response(r.get('data')) if ' ' in message else None

    if message in ['帮助文档', 'help', '帮助', '使用说明']:
        return text_response('https://k.mygo.fun/#/tsugu')



from .utils import *


def player_router(text, default_servers, server):
    return v2api_from_backend('player', text, default_servers, server)


def card_router(text, default_servers, server):
    return v2api_from_backend('card', text, default_servers, server)


def gacha_router(text, default_servers, server):
    return v2api_from_backend('gachaSimulate', text, default_servers, server)


def event_router(text, default_servers, server):
    return v2api_from_backend('event', text, default_servers, server)


def song_router(text, default_servers, server):
    return v2api_from_backend('song', text, default_servers, server)


def song_meta_router(text, default_servers, server):
    return v2api_from_backend('songMeta', text, default_servers, server)


def character_router(text, default_servers, server):
    return v2api_from_backend('character', text, default_servers, server)


def chart_router(text, default_servers, server):
    return v2api_from_backend('chart', text, default_servers, server)


def ycxall_router(text, default_servers, server):
    return v2api_from_backend('ycxAll', text, default_servers, server)


def ycx_router(text, default_servers, server):
    return v2api_from_backend('ycx', text, default_servers, server)


def lsycx_router(text, default_servers, server):
    return v2api_from_backend('lsycx', text, default_servers, server)


def ycm_router(text, default_servers, server):
    return v2api_from_backend('ycm', text, default_servers, server)


def card_illustration_router(text, default_servers, server):
    return v2api_from_backend('cardIllustration', text, default_servers, server)


def bind_player_request_router(platform, user_id, server, status):
    return bind_player_request(platform, user_id, server, status)


def bind_player_verification_router(platform, user_id, server, player_id, status):
    return bind_player_verification(platform, user_id, server, player_id, status)


def set_car_forward_router(platform, user_id, status):
    return set_car_forward(platform, user_id, status)


def set_default_server_router(platform, user_id, text):
    return set_default_server(platform, user_id, text)


def set_server_mode_router(platform, user_id, text):
    return set_server_mode(platform, user_id, text)




import requests
import re
from .config import tsugu_config


def text_response(string):
    return [{"type": "string", "string": string}]


def server_exists(server):
    if server or server == 0:
        return True
    return False


def requests_post(url, data):
    try:
        if tsugu_config.use_proxies:
            response = requests.post(url, json=data, proxies=tsugu_config.proxies)
        else:
            response = requests.post(url, json=data)
        return response.json()
    except Exception as e:
        print(f"发生异常: {e}")
        return text_response(f"发生异常: {e}")


def get_user_data(platform: str, user_id: str):
    data = {
        'platform': platform,
        'user_id': user_id
    }
    result = requests_post(f"{tsugu_config.user_data_backend}/user/getUserData", data)
    return result


def bind_player_request(platform: str, user_id: str, server: int, bind_type: bool):
    data = {
        'platform': platform,
        'user_id': user_id,
        'server': server,
        'bindType': bind_type  # 布尔，表示绑定还是解绑
    }
    result = requests_post(f"{tsugu_config.user_data_backend}/user/bindPlayerRequest", data)
    return result


def bind_player_verification(platform: str, user_id: str, server: int, player_id: str, bind_type: bool):
    data = {
        'platform': platform,
        'user_id': user_id,
        'server': server,
        'playerId': player_id,
        'bindType': bind_type  # 布尔，表示绑定还是解绑
    }
    result = requests_post(f"{tsugu_config.user_data_backend}/user/bindPlayerVerification", data)
    return result


def v2api_from_backend(api, text, default_servers=None, server=3):
    if default_servers is None:
        default_servers = [3, 0]
    path = f"/v2/{api}"
    data = {
        "default_servers": default_servers,
        "server": server,
        "text": text,
        "useEasyBG": tsugu_config.use_easy_bg,
        "compress": tsugu_config.compress
    }
    res = requests_post(f"{tsugu_config.backend}{path}", data)
    return res


def v2_api_command(message, command_matched, api, platform, user_id, channel_id):
    text = message[len(command_matched):].strip()

    if api in ['cardIllustration', 'ycm']:  # 无需验证server信息
        return v2api_from_backend(api, text)

    if api == 'gachaSimulate':
        if channel_id in tsugu_config.ban_gacha_simulate_group_data:
            return text_response('此群禁止使用模拟抽卡功能')

    # 获取用户数据
    user_data = get_user_data(platform, user_id)
    # print(user_data)
    try:
        if user_data['status'] != 'success':
            return text_response('获取用户数据失败')
        return v2api_from_backend(api, text, user_data['data']['default_server'], user_data['data']['server_mode'])
    except:
        return text_response('获取用户数据失败')


def player_status(user_id, platform, server=None):
    user_data = get_user_data(platform, user_id)
    if user_data['status'] != 'success':
        return text_response('获取用户数据失败')
    if server is None:
        server = user_data['data']['server_mode']
    player_id = user_data['data']['server_list'][server]['playerId']
    if player_id == 0:
        return text_response(f'未绑定玩家，请使用 绑定玩家 {tsugu_config.server_list[server]} 进行绑定')
    return v2api_from_backend('player', str(player_id), server=server)


def set_car_forward(platform, user_id, status):
    data = {
        'platform': platform,
        'user_id': user_id,
        'status': status
    }
    result = requests_post(f"{tsugu_config.user_data_backend}/user/changeUserData/setCarForwarding", data)
    return result


def set_default_server(platform, user_id, text):
    data = {
        'platform': platform,
        'user_id': user_id,
        'text': text
    }
    result = requests_post(f"{tsugu_config.user_data_backend}/user/changeUserData/setDefaultServer", data)
    return result


def set_server_mode(platform, user_id, text):
    data = {
        'platform': platform,
        'user_id': user_id,
        'text': text
    }
    result = requests_post(f"{tsugu_config.user_data_backend}/user/changeUserData/setServerMode", data)
    return result


def submit_car_number_msg(message, user_id, platform):
    # 检查car_config['car']中的关键字
    for keyword in tsugu_config.car_config["car"]:
        if str(keyword) in message:
            break
    else:
        return False
    # 检查car_config['fake']中的关键字
    for keyword in tsugu_config.car_config["fake"]:
        if str(keyword) in message:
            return False
    pattern = r"^\d{5}(\D|$)|^\d{6}(\D|$)"
    if not re.match(pattern, message):
        return False
    # 获取用户数据
    user_data = get_user_data(platform, user_id)
    if not user_data['data']['car']:
        return True
    try:
        car_id = message[:6]
        if not car_id.isdigit() and car_id[:5].isdigit():
            car_id = car_id[:5]
        # 构建URL
        url = f"https://api.bandoristation.com/index.php?function=submit_room_number&number={car_id}&user_id={user_id}&raw_message={message}&source={tsugu_config.token_name}&token={tsugu_config.bandori_station_token}"
        # 发送请求
        response = requests.get(url)
        if response.status_code != 200:
            print(f"[Tsugu] 提交车牌失败，HTTP响应码: {response.status_code}")
            return True  # 虽然提交失败，但是确定了是车牌消息
        return True
    except Exception as e:
        print(f"[Tsugu] 发生异常: {e}")
        return True  # 虽然提交失败，但是确定了是车牌消息


def match_command(message, cmd_dict):
    for command, api_value in cmd_dict.items():
        if message.startswith(command):
            return command, api_value
    return None, None


def load_commands_from_config(data):
    # 初始化一个空字典来存储命令到操作的映射
    cmd_dict = {}
    for item in data:
        api = item['api']
        for command_name in item['command_name']:
            cmd_dict[command_name] = api
    return cmd_dict


def query_server_info(msg):
    # print(f"请求服务器信息: {msg}")
    try:
        # 假设 _post 是已定义的发送POST请求的函数
        r = requests_post(tsugu_config.utils_backend + '/v2/utils', {'text': msg})
        if [{'type': 'string', 'string': '发生异常'}] == r:
            return None
        if not r['servers']:
            return None
        server = r['servers'][0]
        # 根据需要处理并返回服务器信息
        return server  # 或者根据需要格式化返回的信息
    except Exception as e:
        raise e




from tsugu import tsugu_config, tsugu_bot, gacha_router
import unittest


class TestTsugu(unittest.TestCase):

    def test_tsugu(self):
        def show_back_msg(data):
            import base64
            from PIL import Image
            import io

            if not data:
                return "[无反馈数据]"

            for item in data:
                if item["type"] == "string":
                    print(f"[文字信息]\n{item['string']}")
                elif item["type"] == "base64":
                    image_data = base64.b64decode(item["string"])
                    print(f"[图像大小: {len(image_data) / 1024:.2f}KB]")
                    Image.open(io.BytesIO(image_data)).show()
                else:
                    print(item)

        self.subTest(
            tsugu_config.config_docs()
        )
        tsugu_config.backend = 'http://127.0.0.1:3000'
        tsugu_config.utils_backend = 'http://127.0.0.1:3000'
        self.subTest(
            show_back_msg(
                gacha_router('10', [3, 0], 0),
            )
        )
        self.subTest(
            show_back_msg(
                tsugu_bot('查卡 ', '114514', 'red', '666808414'),
            )
        )


if __name__ == '__main__':
    unittest.main()





from .bot import tsugu_bot

from .utils import tsugu_config

from .router import player_router
from .router import card_router
from .router import gacha_router
from .router import event_router
from .router import song_router
from .router import song_meta_router
from .router import character_router
from .router import chart_router
from .router import ycxall_router
from .router import ycx_router
from .router import lsycx_router
from .router import ycm_router
from .router import card_illustration_router
from .router import bind_player_request_router
from .router import bind_player_verification_router
from .router import set_car_forward_router
from .router import set_default_server_router
from .router import set_server_mode_router


